/* Synth - Erzeugen von Signalen fuer AEB750 Autogas-Umschalter     */
/*                                                                  */
/* (c) C.Schirp  released for non-commercial use                    */
/*                                                                  */
/* V1.0 Erstausgabe                                                 */
/*                                                                  */
/*                                                                  */


#include <c:\projekte\picsel\cc5\16F628.h>
#include <c:\projekte\picsel\cc5\int16CXX.h>
#include <c:\projekte\picsel\cc5\inline.h>

#pragma config =0x3F18 

// Standard ohne ext reset 0x3F18
// Standard mit ext Reset  0x3F38  

/* Port- Defines */

#define TX_PIN	RA7
#define LED_R   RA6
#define LED_G   RA4
#define BIT_13	RA5

/* variablen */
bank0 uns16 Dip_new;	// alte Schalterstellung
bank0 uns16 Dip_old;	// neue Schalterstellung


/* defines fuer die SW */

#define TICKTIME 666

/*------------------------------------------------------------------*/
/*   II  N   N  II TTTTT                                            */
/*   II  NN  N  II   T                                              */
/*   II  N N N  II   T                                              */
/*   II  N  NN  II   T                                              */
/*   II  N   N  II   T                                              */
/*------------------------------------------------------------------*/

void init (void)
{
	/* PORT A, B Config */
	CMCON = 0x07;		// kein Komparator
	
	TRISA = 0b0010.1111; 	// 4,7,6 Ausgnge,LO Eingaenge 	
	TRISB = 0b1111.1111;	// alles Eingaenge
	
	PORTA = 0b1101.0000;	// HINT: erst Direction, dann Wert!
	PORTB = 0x00;	

	/*sonstiges*/
	T1CON = 0x00;		// Timer disabled
	OPTION_REG = 0x04;	/* pull-up portB, falling edge int, no WDT prescaler, prescale=32 */
	INTCON = 0; 		// reset and disable all interrupts by default

	/* Variablen initialisieren */
	Dip_old=0;
}

void Lese_Dipswitch(void)
{
#pragma updateBank 0 /* OFF, all cores */

	Dip_new.low8 = PORTB;	// Bits 0..7
	Dip_new.high8 = PORTA;
	Dip_new.high8 &= 0x0F;	// Bits 8..11
	Dip_new.12 = BIT_13;		// Bit 12
	Dip_new.13 = 1;				// Bit 13 (startbit) immer an
	
#pragma updateBank 1 /* ON,  all cores */
}

void Delay1ms(uns16 ms)
{
	while(ms)			// Schleife verlassen wenn ms=0 ist
	{
	    OPTION = 2; 		// Vorteiler auf 8 einstellen
	    TMR0 = 131; 		// 125 * 8 = 1000 (= 1 ms)
	    while (TMR0);		// abwarten einer Milisekunde
	    ms--;			// "ms" mit jeder Milisekunde ernidrigen
	}
}

void Send_Msg(void)
{
#pragma updateBank 0 /* OFF, all cores */

	uns8 bitcount;
	
	LED_G = 0;	// Send Msg
	
	TX_PIN = 0;	// Start 3ms Low
	Delay1ms(3);
	
	for (bitcount=0; bitcount<14; bitcount++)
	{
		TX_PIN = 1;
		
		if (Dip_new & 0x2000)
		{
			Delay1ms(4);	// Eins = 4ms high
		}
		else
		{
			Delay1ms(2);	// Null = 2ms high
		}
		
		Dip_new <<=1;
		
		TX_PIN = 0;
		Delay1ms(3);
	}
	
	TX_PIN = 1;	// Idle-Pegel
	
	LED_G = 1;	// Send Msg

#pragma updateBank 1 /* ON,  all cores */	
}



/*------------------------------------------------------------------*/
/*     M     M    A    II  N   N                                    */
/*     MM   MM   A A   II  NN  N                                    */
/*     M M M M  A   A  II  N N N                                    */
/*     M  M  M  AAAAA  II  N  NN                                    */
/*     M     M  A   A  II  N   N                                    */
/*------------------------------------------------------------------*/

void main (void)
{
	uns16 ticks;
	uns16 helper;
	init();
	
	ticks = 0; // Einmal sofort nach dem Start senden
	
	Delay1ms(200);	// Erstmal initialisieren lassen

#pragma updateBank 0 /* OFF, all cores */
  
	while (1)
	{
		LED_R ^=1;	// Togle lifeindicator
		
		Lese_Dipswitch();
		
		// Neue Schalterstellung oder 6 sek abgelaufen
		helper = Dip_new-Dip_old;
		
		Dip_old = Dip_new;	// alte Schalterstellung merken
		
		if (helper || (!ticks))
		{
			Send_Msg();
			ticks = TICKTIME;	// 6 sekunden
		}
		
		Delay1ms(9);	// Abstand zwei Botschaften
		
		ticks--;	// Ticktimer zaehlen
	}; 
#pragma updateBank 1 /* ON, all cores */
}